defmodule DecentralisedBookIndex.Metadata.Author do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer,
    extensions: [AshJsonApi.Resource],
    authorizers: [Ash.Policy.Authorizer]

  require Ash.Query
  alias DecentralisedBookIndex.Metadata

  @brief_description_length 20
  @brief_description_max_length @brief_description_length + 3

  json_api do
    type "author"
  end

  postgres do
    table "authors"
    repo DecentralisedBookIndex.Repo

    custom_indexes do
      index "name gin_trgm_ops", name: "author_name_gin_index", using: "GIN"
    end
  end

  actions do
    defaults [:read, :destroy]

    create :create do
      primary? true
      accept [:name, :description, :avatar_url, :author_alias_registry_id]

      change fn changeset, context ->
        actor = Map.get(context, :actor, nil)

        registry_id = Ash.Changeset.get_attribute(changeset, :author_alias_registry_id)

        if registry_id == nil do
          registry =
            DecentralisedBookIndex.Metadata.create_author_alias_registry!(actor: actor)

          Ash.Changeset.force_change_attribute(changeset, :author_alias_registry_id, registry.id)
        else
          changeset
        end
      end
    end

    create :add_author_to_related_alias_registry do
      accept [:name, :description, :avatar_url]

      argument :related_author_id, :uuid do
        allow_nil? false
      end

      change fn changeset, context ->
        related_author_id = changeset.arguments.related_author_id

        if related_author_id == nil do
          Ash.Changeset.add_error(changeset, :related_author_id, "Related author is empty")
        else
          related_author =
            DecentralisedBookIndex.Metadata.get_author_by_id!(related_author_id)

          Ash.Changeset.force_change_attribute(
            changeset,
            :author_alias_registry_id,
            related_author.author_alias_registry_id
          )
        end
      end
    end

    create :sync_create do
      accept [
        :id,
        :name,
        :description,
        :avatar_url,
        :author_alias_registry_id,
        :inserted_at,
        :updated_at,
        :dbi_server_id
      ]

      change fn changeset, context ->
        registry_id = Ash.Changeset.get_attribute(changeset, :author_alias_registry_id)

        if registry_id == nil do
          {:ok, registry} =
            DecentralisedBookIndex.Metadata.create_author_alias_registry(authorize?: false)

          Ash.Changeset.force_change_attribute(changeset, :author_alias_registry_id, registry.id)
        else
          changeset
        end
      end
    end

    read :by_id do
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end

    read :get_alternative_names do
      argument :author, :struct, allow_nil?: false

      prepare fn query, context ->
        author = query.arguments.author

        DecentralisedBookIndex.Metadata.Author
        |> Ash.Query.filter(
          author_alias_registry_id == ^author.author_alias_registry_id and
            id != ^author.id
        )
      end
    end

    action :get_author_ids, :vector do
      argument :author, :struct, allow_nil?: false

      run fn input, _ ->
        author = input.arguments.author

        {:ok, ids} =
          DecentralisedBookIndex.Metadata.Author
          |> Ash.Query.filter(author_alias_registry_id == ^author.author_alias_registry_id)
          |> Ash.Query.select([:id])
          |> Ash.read()

        {:ok, Enum.map(ids, & &1.id)}
      end
    end

    read :search do
      argument :name, :ci_string do
        constraints allow_empty?: true
        default ""
      end

      filter expr(contains(name, ^arg(:name)))

      pagination offset?: true, default_limit: 10
    end

    update :sync do
      description "Sync the data for an existing record."
      require_atomic? false

      accept [
        :id,
        :name,
        :description,
        :avatar_url,
        :author_alias_registry_id,
        :inserted_at,
        :updated_at,
        :dbi_server_id
      ]

      change fn changeset, _ ->
        registry_id = Ash.Changeset.get_attribute(changeset, :author_alias_registry_id)

        if registry_id == nil do
          {:ok, registry} = DecentralisedBookIndex.Metadata.create_author_alias_registry()

          Ash.Changeset.force_change_attribute(changeset, :author_alias_registry_id, registry.id)
        else
          changeset
        end
      end
    end

    update :update do
      primary? true
      accept [:name, :description, :avatar_url, :author_alias_registry_id]
      require_atomic? false

      change fn changeset, _ ->
        registry_id = Ash.Changeset.get_attribute(changeset, :author_alias_registry_id)

        if registry_id == nil do
          {:ok, registry} = DecentralisedBookIndex.Metadata.create_author_alias_registry()

          Ash.Changeset.force_change_attribute(changeset, :author_alias_registry_id, registry.id)
        else
          changeset
        end
      end
    end

    update :assign_avatar_image do
      accept [:avatar_url]
    end

    update :assign_dbi_server_id do
      accept [:dbi_server_id]
    end
  end

  policies do
    bypass actor_attribute_equals(:role, :admin) do
      authorize_if always()
    end

    policy action_type(:read) do
      authorize_if always()
    end

    policy action(:get_author_ids) do
      authorize_if always()
    end

    policy action_type(:create) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:update) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:destroy) do
      authorize_if actor_attribute_equals(:role, :admin)
    end
  end

  attributes do
    uuid_primary_key :id, writable?: true

    attribute :name, :string do
      allow_nil? false
      public? true
    end

    attribute :description, :string do
      allow_nil? false
      public? true
    end

    attribute :avatar_url, :string do
      allow_nil? true
      public? true
    end

    timestamps do
      writable? true
      public? true
    end
  end

  relationships do
    belongs_to :dbi_server, Metadata.DBIServer do
      public? true
    end

    belongs_to :author_alias_registry, Metadata.AuthorAliasRegistry do
      attribute_writable? true
      public? true
    end

    has_many :author_role, Metadata.AuthorRole
  end

  calculations do
    calculate :brief_description,
              :string,
              expr(
                if(string_length(description) <= @brief_description_max_length) do
                  description
                else
                  fragment("substr(?, 1, ?)", description, @brief_description_length) <> "..."
                end
              )
  end
end
