defmodule DecentralisedBookIndexWeb.AuthorLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="author-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
          <.input field={@form[:name]} type="text" label="Name" />
          <.input field={@form[:avatar_url]} type="text" label="Cover image url" />
          <.input field={@form[:description]} type="textarea" label="Description" />
          <div>
            <.input field={@form[:author_alias_registry_id]} type="hidden" label="Author alias registry" />
            <.selected_author_alias author_alias_registry_form={@form[:author_alias_registry_id]} notify_component={@myself} />
          </div>
        <% end %>
        <%= if @form.source.type == :update do %>
          <.input field={@form[:name]} type="text" label="Name" />
          <.input field={@form[:avatar_url]} type="text" label="Cover image url" />
          <.input field={@form[:description]} type="textarea" label="Description" />
          <div>
            <.input field={@form[:author_alias_registry_id]} type="hidden" label="Author alias registry" />
            <.selected_author_alias author_alias_registry_form={@form[:author_alias_registry_id]} notify_component={@myself} />
          </div>
        <% end %>

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
          <.cancel_button phx-click="cancel" phx-target={@myself}>
            Cancel
          </.cancel_button>
        </:actions>
      </.simple_form>

      <.modal
        :if={@select_author_alias_open? == true}
        id="select-author-alias-modal"
        show
        on_cancel={JS.push("close-select-author-alias", target: @myself)}
      >
        <.live_component
          id="select-author-alias"
          module={SelectAuthorAlias}
          current_user={@current_user}
          notify_component={@myself}
        />
      </.modal>
    </div>
    """
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign(:select_author_alias_open?, false)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"author" => author_params}, socket) do
    {:noreply, assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, author_params))}
  end

  def handle_event("save", %{"author" => author_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form,
           params: author_params,
           actor: socket.assigns.current_user
         ) do
      {:ok, author} ->
        notify_parent({:saved, author})

        socket =
          socket
          |> put_flash(:info, "Author #{socket.assigns.form.source.type}d successfully")
          |> redirect(to: patch_url(socket.assigns.action, author.id))

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  def handle_event("cancel", _params, socket) do
    author = socket.assigns.author
    author_id = if author == nil, do: nil, else: author.id

    socket =
      socket
      |> push_navigate(to: patch_url(socket.assigns.action, author_id))

    {:noreply, socket}
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(
         %{assigns: %{author: author, alternative_author_id: alternative_author_id}} = socket
       ) do
    alias_id =
      if alternative_author_id == nil do
        nil
      else
        alternative_author = Metadata.get_author_by_id!(alternative_author_id)
        alternative_author.author_alias_registry_id
      end

    form =
      if author do
        AshPhoenix.Form.for_update(author, :update,
          as: "author",
          actor: socket.assigns.current_user
        )
      else
        form =
          AshPhoenix.Form.for_create(Metadata.Author, :create,
            as: "author",
            actor: socket.assigns.current_user
          )
          |> set_author_alias_registry_id(alias_id)
      end

    assign(socket, form: to_form(form))
  end

  defp set_author_alias_registry_id(form, alias_id) do
    if alias_id != nil do
      form
      |> AshPhoenix.Form.update_params(&Map.put(&1, "author_alias_registry_id", alias_id))
    else
      form
    end
  end

  defp patch_url(action, author_id) do
    case action do
      :edit -> ~p"/authors/#{author_id}"
      :new -> ~p"/authors"
      _ -> ~p"/authors"
    end
  end

  def handle_event("select-author-alias", %{"author-alias" => author_alias_id}, socket) do
    IO.inspect("book_editions_registry_id")
    IO.inspect(author_alias_id)

    socket =
      socket
      |> update(:form, fn form ->
        form
        |> AshPhoenix.Form.update_params(&Map.put(&1, "author_alias_registry_id", author_alias_id))
      end)
      |> assign(:select_author_alias_open?, false)

    {:noreply, socket}
  end

  def handle_event("open-select-author-alias", _params, socket) do
    socket =
      socket
      |> assign(:select_author_alias_open?, true)

    {:noreply, socket}
  end

  def handle_event("close-select-author-alias", _params, socket) do
    socket =
      socket
      |> assign(:select_author_alias_open?, false)

    {:noreply, socket}
  end
end
