defmodule DecentralisedBookIndex.Metadata.AuthorTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "authors and registries relationship" do
    test "a new author get new registry by default", %{user: user} do
      assert {:ok, author} =
               Metadata.create_author("Author", "An description", nil, nil, actor: user)

      assert author.author_alias_registry_id != nil
    end

    test "a new author belongs to a registry if specified", %{user: user} do
      assert {:ok, registry} = Metadata.create_author_alias_registry(actor: user)

      assert {:ok, author} =
               Metadata.create_author("Author", "An description", nil, registry.id, actor: user)

      assert author.author_alias_registry_id == registry.id
    end

    test "a new author to a registry via a related author record", %{user: user} do
      {:ok, related_author} =
        Metadata.create_author("Author", "An description", nil, nil, actor: user)

      assert {:ok, author} =
               Metadata.add_author_to_related_alias_registry(
                 "Author2",
                 "An description2",
                 related_author.id,
                 nil,
                 actor: user
               )

      assert related_author.author_alias_registry_id == author.author_alias_registry_id
    end

    test "a new author to a registry via a empty related author record", %{user: user} do
      assert {:error, _} =
               Metadata.add_author_to_related_alias_registry("Author2", "An description2", nil,
                 actor: user
               )
    end
  end

  describe "authors alternatives names" do
    test "new author has no alternatives names", %{user: user} do
      {:ok, author} = Metadata.create_author("Author", "An description", nil, nil, actor: user)
      assert {:ok, alternatives_names} = Metadata.get_author_alternative_names(author)
      assert alternatives_names = []
    end

    test "author has related author so they has one alternative name", %{user: user} do
      {:ok, related_author} =
        Metadata.create_author("Author", "An description", nil, nil, actor: user)

      {:ok, author} =
        Metadata.add_author_to_related_alias_registry(
          "Author2",
          "An description2",
          related_author.id,
          nil,
          actor: user
        )

      assert {:ok, alternatives_names} = Metadata.get_author_alternative_names(author)
      assert alternatives_names = [related_author]
    end
  end

  describe "author's ids" do
    test "the list has the author's id", %{user: user} do
      {:ok, author} = Metadata.create_author("Author", "An description", nil, nil, actor: user)

      assert {:ok, ids} = Metadata.get_author_ids(author, actor: user)
      assert author.id in ids
    end

    test "the list has the aliases' ids", %{user: user} do
      {:ok, author} = Metadata.create_author("Author", "An description", nil, nil, actor: user)

      {:ok, alias1} =
        Metadata.add_author_to_related_alias_registry(
          "Author2",
          "An description2",
          author.id,
          nil,
          actor: user
        )

      {:ok, alias2} =
        Metadata.add_author_to_related_alias_registry(
          "Author3",
          "An description3",
          author.id,
          nil,
          actor: user
        )

      assert {:ok, ids} = Metadata.get_author_ids(author)

      assert author.id in ids
      assert alias1.id in ids
      assert alias2.id in ids
    end
  end

  describe "authors and avatar_url" do
    test "a new author has no avatar by default", %{user: user} do
      assert {:ok, author} =
               Metadata.create_author("Author", "An description", nil, nil, actor: user)

      assert author.avatar_url == nil
    end

    test "a new author has a avatar", %{user: user} do
      avatar_url = "/images/avatar.png"

      assert {:ok, author} =
               Metadata.create_author("Author", "An description", avatar_url, nil, actor: user)

      assert author.avatar_url == avatar_url
    end
  end

  describe "author and DBIServer relationship" do
    test "assign a DBIServer", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      assert {:ok, author} =
               Metadata.create_author("Author", "An description", nil, nil, actor: user)
      assert {:ok, author} = Metadata.assign_author_dbi_server(author, server.id, actor: user)
      assert author.dbi_server_id == server.id
    end
  end

  describe "calculate a brief description" do
    test "a short description is already enough" do
      description = "Amazing!"
      assert description == Ash.calculate!(Metadata.Author, :brief_description,
                                           refs: %{description: description})
    end

    test "a long description is too much" do
      description = "Etiam laoreet quam sed arcu. Donec at pede. Pellentesque tristique imperdiet tortor."
      brief = "Etiam laoreet quam s..."

      assert String.length(brief) == 23
      assert brief == Ash.calculate!(Metadata.Author, :brief_description,
                                     refs: %{description: description})
    end
  end
end
