defmodule DecentralisedBookIndex.Sync.DataTransformers.AuthorSyncTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Sync.AuthorSync
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "sync author transformations" do
    test "a new author will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      author = %{
        id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
        name: "Author",
        description: "Something",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: nil,
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      {:ok, inserted_at, 0} = DateTime.from_iso8601(author[:inserted_at])
      {:ok, updated_at, 0} = DateTime.from_iso8601(author[:updated_at])

      assert :ok = AuthorSync.create_update(author, server.id)
      assert {:ok, saved_author} = Metadata.get_author_by_id(author.id)

      author =
        author
        |> Map.replace(:inserted_at, inserted_at)
        |> Map.replace(:updated_at, updated_at)
        |> Map.delete(:author_alias_registry_id)

      assert get_submap(saved_author, author) == author
      assert nil != saved_author.author_alias_registry_id
      assert server.id == saved_author.dbi_server_id
    end

    test "a new author has the same alias registry" do
      server = generate(dbi_server(url: @test_server_endpoint))

      author = %{
        id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
        name: "Author",
        description: "Something",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: "1320a6fe-b311-45ac-bbaa-39cf29c44624",
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      {:ok, inserted_at, 0} = DateTime.from_iso8601(author[:inserted_at])
      {:ok, updated_at, 0} = DateTime.from_iso8601(author[:updated_at])

      assert :ok = AuthorSync.create_update(author, server.id)
      assert {:ok, saved_author} = Metadata.get_author_by_id(author.id)

      author =
        author
        |> Map.replace(:inserted_at, inserted_at)
        |> Map.replace(:updated_at, updated_at)

      assert get_submap(saved_author, author) == author
      assert nil != saved_author.author_alias_registry_id
      assert server.id == saved_author.dbi_server_id
    end

    test "update an existing author", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, author} = Metadata.create_author("Author", "An description", nil, nil, actor: user)

      author_attrs = %{
        id: author.id,
        name: "Author2",
        description: "Something2",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: nil
      }

      assert :ok = AuthorSync.create_update(author_attrs, server.id)
      assert {:ok, saved_author} = Metadata.get_author_by_id(author.id)

      assert author = saved_author
    end

    test "the local existing author wouldn't updated", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, author} = Metadata.create_author("Author", "An description", nil, nil, actor: user)

      author_attrs = %{
        id: author.id,
        name: "Author",
        description: "Something2",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: nil,
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      assert :ok = AuthorSync.create_update(author_attrs, server.id)
      assert {:ok, saved_author} = Metadata.get_author_by_id(author.id)

      assert get_submap(saved_author, author_attrs) != author_attrs
      assert saved_author.name == "Author"
      assert saved_author.dbi_server_id == nil
    end

    test "old author wouldn't be sync", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, author} = Metadata.create_author("Author", "An description", nil, nil, actor: user)
      {:ok, author} = Metadata.assign_author_dbi_server(author, server.id, actor: user)

      author_attrs = %{
        id: author.id,
        name: "Author",
        description: "Something2",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: nil,
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      assert :ok = AuthorSync.create_update(author_attrs, server.id)
      assert {:ok, saved_author} = Metadata.get_author_by_id(author.id)

      assert get_submap(saved_author, author_attrs) != author_attrs
      assert saved_author.name == "Author"
    end
  end
end
